/*                                                           
HTML.cc
*/

#include "HTML.h"
extern Language *L;

HTML::HTML (FILE *FR)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = FR;
  iout = -1;
  OneSocket = NULL;
  XBuffer = XString ("");
  }
  

HTML::HTML (int FR)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = NULL;
  iout = FR;
  OneSocket = NULL;
  XBuffer = XString ("");  
  }
   
HTML::HTML (XSocket *aXS)
  {
  MAXMSGSFORPAGE = DEFOPT_MSGSPERINDEXPAGE;
  fout = NULL;
  iout = -1;
  OneSocket = aXS;
  XBuffer = XString ("");
  }
 
HTML::~HTML()
  {
  }  
  
void HTML::setMAXMSGSFORPAGE (long maxmsgs)
  {
  MAXMSGSFORPAGE = maxmsgs;
  }          

void HTML::WRITE (char *fmt, ...)
  {
  TBuffer linea;
  va_list args;
  va_start (args, fmt);
  if (strlen(fmt) > CMAXBUFFER) {fmt[CMAXBUFFER] = '\0';}
  vsprintf (linea, fmt, args);  
  if (iout != -1) write (iout, linea, strlen(linea));
  else if (OneSocket != NULL) 
    {    
    OneSocket->Puts (linea);
    }
  else {fprintf (fout, linea); fflush (fout);}
  va_end (args);      
  }  

void HTML::pcWRITE (char *pc)
  {
  if (iout != -1) write (iout, pc, strlen(pc));
  else if (OneSocket != NULL) 
    {
    OneSocket->Puts (pc);
    }
  else {fprintf (fout, pc); fflush (fout);}
  }  
  
void HTML::pcWRITEBuffer (char *pc, xulong len)
  {
  if (pc == NULL) {LOG("Null in pcWRITEBuffer"); return;}
  if (iout != -1) write (iout, pc, (uint)len);
  else if (OneSocket != NULL) 
    {
    OneSocket->Write (pc, (uint)len);    
    }
  else {write (fileno(fout), pc, (uint)len); fflush (fout);}
  }    
  
void HTML::printCookie (const char *cookieline)
  {
  if ((cookieline != NULL) && (strcmp(cookieline, "") != 0)) 
    {
    WRITE ("Set-Cookie: %s\n", cookieline);
    }
  }   
  
void HTML::Head (const char *contenttype, const char *name, const char *filename)
  {
  if ((name     != NULL) && (strcmp(name, "")     != 0)) {WRITE ("Name: %s\n", name);}
  if ((filename != NULL) && (strcmp(filename, "") != 0)) {WRITE ("Filename: %s\n", filename);}
  WRITE ("Content-type: %s\n", contenttype);  
  WRITE ("\n");
  } 
  
void HTML::HeadInline (const char *contenttype, const char *filename, xulong len)
  {        
  if ((strcmp (filename, "") != 0) && (filename != NULL) && (len > 0))
    {
    WRITE ("Content-Disposition: inline; filename=%s\n", filename);
    WRITE ("Content-Length: %ld\n", len);
    }
  WRITE ("Content-type: %s\n", contenttype);  
  WRITE ("\n");
  }   

void HTML::EmulatePageNotFound (void)
  {
  //WRITE ("HTTP/1.0 404 Not Found\n");
  WRITE ("Connection: close\n");
  WRITE ("Status: 404 Not Found\n");
  WRITE ("Content-type: text/html\n");
  WRITE ("\n");
  WRITE ("%s\n", HTML_DOCTYPE);
  WRITE ("<HTML><HEAD>\n");
  WRITE ("<TITLE>404 Not Found</TITLE>\n");
  WRITE ("</HEAD><BODY>\n"); 
  WRITE ("<H1>Not Found</H1>\n");
  WRITE ("The requested URL was not found on this server.<P>\n");
  WRITE ("</BODY></HTML>\n");  
  }
  
void HTML::HTMLBegin (void)
  {
  WRITE ("%s\n<HTML>\n", HTML_DOCTYPE);
  }  
  
void HTML::Title (char *title, char *bgcolor, char *txtcolor, char *img_bg, bool include_js, int refresh_time, char *newurl)
  {
  WRITE ("<HEAD>\n");
  WRITE ("<TITLE>%s</TITLE>\n", title);
  if (include_js == true)
    {
    //Here we can put JavaScripts
    }
       
  if ((refresh_time > 0) && (newurl != NULL))
    {
    //Time arrives in minutes and transform to seconds
    //DEBUG WRITE ("<META HTTP-EQUIV=\"refresh\" CONTENT=\"%d;URL=%s\">\n", refresh_time, newurl);  
    WRITE ("<META HTTP-EQUIV=\"refresh\" CONTENT=\"%d;URL=%s\">\n", refresh_time * 60, newurl);
    }
  
  WRITE ("</HEAD>\n");
  if ((strcmp (img_bg, " ") != 0) && (img_bg != NULL))
    {
    WRITE ("<BODY background=\"%s\" text=\"%s\" bgcolor=\"%s\" link=\"#3333CC\" vlink=\"#3333CC\" alink=\"#3333CC\">\n", 
           img_bg, txtcolor, bgcolor);            
    }
  else
    {
    WRITE ("<BODY text=\"%s\" bgcolor=\"%s\" link=\"#3333CC\" vlink=\"#3333CC\" alink=\"#3333CC\">\n", 
            txtcolor, bgcolor);            
    }
  }  
  
void HTML::HTMLEnd (char *bgtablecopyrightcolor)
  {
  Linea ();  
  BeginTable (0, 2, 1, 100);
  if (bgtablecopyrightcolor == NULL) {WRITE ("<TR bgcolor=\"%s\">", DEFCOLOR_BGTABLECOPYRIGHT);}
  else {WRITE ("<TR bgcolor=\"%s\">", bgtablecopyrightcolor);}
  BeginTDColSpanCenter (7); Small (L->get (L_COPYRIGHT)); EndTD();
  EndTR();
  EndTable();
  if (SHOW_VALIDATE_HTML_URL == 1) {WRITE ("<BR><A HREF=\"http://www.htmlhelp.com/cgi-bin/validate.cgi?url=referer&amp;input=yes\">Validate HTML code</A>\n");}
  WRITE ("</BODY>\n");
  WRITE ("</HTML>\n"); 
  }                  
  
void HTML::BR (void)
  {
  WRITE ("<BR>\n");
  }                    

void HTML::JavaScript (char *codigo)
  {
  WRITE ("<SCRIPT LANGUAGE=\"JavaScript\" TYPE=\"text/javascript\">\n");
  WRITE ("<!--\n");
  WRITE ("%s", codigo);
  WRITE ("\n// -->\n");
  WRITE ("</SCRIPT>\n");
  }

void HTML::Small (char *linea)
  {
  WRITE ("<SMALL>%s</SMALL>\n", linea);
  }

void HTML::H1 (char *linea)
  {
  WRITE ("<H1>%s</H1>\n", linea);
  }
  
void HTML::H2 (char *linea)
  {
  WRITE ("<H2>%s</H2>\n", linea);
  }

void HTML::H3 (char *linea)
  {
  WRITE ("<H3>%s</H3>\n", linea);
  }  
  
void HTML::H4 (char *linea)
  {
  WRITE ("<H4>%s</H4>\n", linea);
  }

void HTML::H5 (char *linea)
  {
  WRITE ("<H5>%s</H5>\n", linea);
  }  
  
void HTML::BeginTable (int border, int cellpadding, int cellspacing, int widthpercent)
  {
  WRITE ("<TABLE cellpadding=\"%d\" cellspacing=\"%d\" width=\"%d%%\" BORDER=\"%d\">\n", cellpadding, cellspacing, widthpercent, border);
  }

void HTML::EndTable (void)
  {
  WRITE ("</TABLE>\n");
  }

void HTML::BeginTR (void)
  {
  WRITE ("<TR>\n");
  }

void HTML::BeginTR (char *bgcolor)
  {
  WRITE ("<TR bgcolor=\"%s\">\n", bgcolor);
  }

void HTML::EndTR (void)
  {
  WRITE ("</TR>\n");
  }

void HTML::BeginTD (void)
  {
  WRITE ("<TD>\n");
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTDColSpan (int acolspan)
  {
  WRITE ("<TD colspan=\"%d\">\n", acolspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTDColSpanCenter (int acolspan)
  {
  WRITE ("<TD align=\"center\" colspan=\"%d\">\n", acolspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }  
  
void HTML::BeginTDCenter (void)
  {
  WRITE ("<TD align=\"center\">\n");
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTD (char *bgcolor)
  {
  WRITE ("<TD bgcolor=\"%s\">\n", bgcolor);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }      

void HTML::BeginTD (char *bgcolor, char *align)
  {
  WRITE ("<TD bgcolor=\"%s\" align=\"%s\">\n", bgcolor, align);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }

void HTML::BeginTD (char *bgcolor, char *align, int colspan)
  {
  WRITE ("<TD bgcolor=\"%s\" align=\"%s\" colspan=\"%d\">\n", bgcolor, align, colspan);
  WRITE ("<FONT size=2 face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }

void HTML::BeginTD (int widthpercent)
  {
  WRITE ("<TD width=\"%d%%\">\n", widthpercent);    
  WRITE ("<FONT size=\"2\" face=\"%s\" color=\"%s\">\n", TABLEFACEFONT, BLACK);
  }
  
void HTML::BeginTD (bool nowrap, int widthpercent, int rowspan, char *facefont, int sizefont, char *align)
  {
  if (nowrap == true)
    {
    WRITE ("<TD nowrap width=\"%d%%\" rowspan=\"%d\" align=\"%s\">\n", widthpercent, rowspan, align);
    }
  else
    {
    WRITE ("<TD width=\"%d%%\" rowspan=\"%d\" align=\"%s\">\n", widthpercent, rowspan, align);    
    }
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", sizefont, facefont, BLACK);
  }

void HTML::BeginTD (int widthpercent, char *facefont, int sizefont, char *align)
  {
  WRITE ("<TD width=\"%d%%\" align=\"%s\">\n", widthpercent, align);      
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", sizefont, facefont, BLACK);
  }

void HTML::EndTD (void)
  {
  WRITE ("</FONT></TD>\n");
  }

void HTML::EmptyCellx1 (void)
  {
  BeginTD (); WRITE(HTML_HSP); EndTD();
  }
  
void HTML::EmptyCellx2 (void)
  {
  BeginTD (); WRITE(HTML_HSP); EndTD();
  BeginTD (); WRITE(HTML_HSP); EndTD();
  }                

void HTML::BeginBold (void)
  {
  WRITE ("<B>\n");
  }

void HTML::EndBold (void)
  {
  WRITE ("</B>\n");
  }

void HTML::BeginItalic (void)
  {
  WRITE ("<I>\n");
  }

void HTML::EndItalic (void)
  {
  WRITE ("</I>\n");
  }
void HTML::BeginUnderline (void)
  {
  WRITE ("<U>\n");
  }

void HTML::EndUnderline (void)
  {
  WRITE ("</U>\n");
  }

void HTML::BeginFont (char *face, int size, char *color)
  {
  WRITE ("<FONT size=\"%d\" face=\"%s\" color=\"%s\">\n", size, face, color);
  }
  
void HTML::EndFont (void)
  {
  WRITE ("</FONT>\n");
  }        

void HTML::Label (char *linea)
  {
  WRITE ("<B>%s</B>\n", linea);
  }  

void HTML::RedLabel (char *linea)
  {
  WRITE ("<FONT color=\"#FF0000\"><B>%s</B></FONT>\n", linea);
  }

void HTML::GreenLabel (char *linea)
  {
  WRITE ("<FONT color=%s><B>%s</B></FONT>\n", GREEN, linea);
  }

void HTML::Image (char *urlimage, char *label)
  {
  WRITE ("<IMG BORDER=0 SRC=\"%s\" ALT=\"%s\">\n", urlimage, label);
  }
  
void HTML::URL (char *caption, char *url)
  {
  WRITE ("<A HREF=\"%s\">%s</A>\n", url, caption);
  }  
                            
void HTML::URLIMAGE (char *label, char *url, char *urlimage)
  {
  WRITE ("<A HREF=\"%s\"><IMG BORDER=0 SRC=\"%s\" ALT=\"%s\"></A>\n", url, urlimage, label);
  }    

void HTML::BeginPre ()
  {
  WRITE ("<PRE>\n");
  }
  
void HTML::EndPre ()
  {
  WRITE ("</PRE>\n");
  }
   
void HTML::BeginCenter ()
  {
  WRITE ("<CENTER>\n");
  }
 
void HTML::EndCenter ()
  {
  WRITE ("</CENTER>\n");
  } 

void HTML::PrintString (char *linea)
  {
  WRITE ("%s<BR>\n", linea);
  }
  
void HTML::pr (char *linea)
  {
  WRITE ("%s\n", linea);
  }
        
void HTML::Redirect (char *newurl)
  {
  WRITE ("Status: 302 Moved\n");
  WRITE ("Location: %s\n", newurl);
  WRITE ("Content-Type: text/html\n\n");
  }  
  
void HTML::BeginNOBR (void)
  {
  WRITE ("<NOBR>\n");
  }
  
void HTML::EndNOBR(void)
  {
  WRITE ("</NOBR>\n");
  }        
  
void HTML::Linea ()
  {
  WRITE ("<HR noshade size=2>\n");
  }
  
void HTML::Error (char *error)
  {
  WRITE ("<p><h1><FONT color=\"#FF0000\">Error:</font> %s</h1><p>\n", error);
  } 

void HTML::Msg (char *msg)
  {
  WRITE ("<p><h1>%s</h1><p>\n", msg);
  } 
 
void HTML::ErrorPage (char *error)
  {                    
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (error, WHITE, BLACK, DEFIMG_BG, false, 0, NULL);
  WRITE ("<p><h1><FONT color=\"%s\">Error:</font> %s</h1><p>\n", RED, error);
  HTMLEnd(NULL);      
  } 

void HTML::ErrorPageBack (char *stlang, char *user, char *connid, UserOptions *uo, char *error, int way)
  {                    
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (error, uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  switch (way)
    {       
    case 1:
      {
      BeginTable (0, 0, 0, 100);
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
             uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_INDEXMAILBOX), L->get(L_INDEXMAILBOX2));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();        
      EmptyCellx2();
      EndTR();
      EndTable (); 
      break;
      }
    case 2:
      {
      BeginTable (0, 0, 0, 100);
      BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_forward/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
             uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(L_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();      
      EmptyCellx2();  
      EndTR();
      EndTable (); 
      break;
      }      
    }
  WRITE ("<p><h1><FONT color=\"%s\">Error:</font> %s</h1><p>\n", RED, error);
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }                                                       

void HTML::MsgPage (char *msg, char *lang)
  {                       
  Head ("text/html", "", "");  
  HTMLBegin ();  
  Title (msg, WHITE, BLACK, DEFIMG_BG, false, 0, NULL);
  WRITE ("<p><h1><FONT color=%s>%s</font></h1><p>\n", GREEN, msg);
  WRITE ("<p><A HREF=\"%s?lang=%s\">%s</A>", PATH_POSTMAN, lang, L->get(MSG_CANRECONNECTHERE));
  HTMLEnd(NULL); 
  } 

void HTML::OkPage (char *stlang, char *user, char *connid, UserOptions *uo, char *okmsg)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (okmsg, uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  WRITE ("<BR><FONT color=%s size=5><B>%s</B></FONT><BR><BR>\n", GREEN, okmsg);
  WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
         uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_INDEXMAILBOX), L->get(L_INDEXMAILBOX2));
  WRITE ("</A>");
  Label (L->get(L_PRESS_TO_INDEX));
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }  
  
void HTML::Ok (char *linea)
  {
  WRITE ("<p><h1><FONT color=%s>%s</font></h1><p>\n", GREEN, linea);
  } 
            
void HTML::BeginFormPost (char *action)
  {
  WRITE ("<FORM ACTION=\"%s\" METHOD=\"POST\">\n", action);
  } 
  
void HTML::BeginFormPostMultipartFormData (char *action)
  {
  WRITE ("<FORM ACTION=\"%s\" METHOD=\"POST\" ENCTYPE=\"multipart/form-data\">\n", action);
  }                          
                             
void HTML::BeginFormGet (char *action)
  {
  WRITE ("<FORM ACTION=\"%s\" METHOD=\"GET\">\n", action);
  }   
  
void HTML::EndForm ()
  {
  WRITE ("</FORM>\n");
  }   
    
void HTML::FormBeginSelect (char *name)
  {
  WRITE ("<SELECT SIZE=1 NAME=\"%s\">\n", name);
  }                                  
  
void HTML::FormAddOptionSelect (bool selected, const char *value, const char *label)
  {
  if (selected == true) 
    {
    WRITE ("<OPTION selected VALUE=\"%s\">%s\n", value, label);
    }
  else
    {
    WRITE ("<OPTION VALUE=\"%s\">%s\n", value, label);
    }
  }

void HTML::FormEndSelect ()
  {
  WRITE ("</SELECT>\n");
  }                                       

void HTML::FormHiddenQuoted (char *name, char *value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s%s%s\">\n", name, HTML_DCOMILLAS, value, HTML_DCOMILLAS);
  }
   
void HTML::FormHidden (char *name, char *value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%s\">\n", name, value);
  } 

void HTML::FormHidden (char *name, int value)
  {
  WRITE ("<INPUT TYPE=HIDDEN NAME=\"%s\" VALUE=\"%d\">\n", name, value);
  } 

void HTML::FormEdit (int max, int size, char *name, char *value)
  {
  WRITE ("<INPUT TYPE=TEXT NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\">\n", 
           name, max, size, value);
  }  

void HTML::FormFile (int max, int size, char *name, char *value, char *accept)
  {
  WRITE ("<INPUT TYPE=FILE NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\" ACCEPT=\"%s\">\n", 
           name, max, size, value, accept);
  }

void HTML::FormTextArea (int rows, int cols, char *name, char *value)
  {
  WRITE ("<TEXTAREA NAME=\"%s\" ROWS=\"%d\" COLS=\"%d\">\n", name, rows, cols);
  pcWRITEBuffer (value, strlen (value));
  WRITE ("</TEXTAREA>\n");
  }  
   
void HTML::FormPassword (int max, int size, char *name, char *value)
  {
  WRITE ("<INPUT TYPE=PASSWORD NAME=\"%s\" MAXLENGTH=\"%d\" SIZE=\"%d\" VALUE=\"%s\">\n", 
           name, max, size, value);
  }  
  
void HTML::FormCheckbox (char *name, char *value, bool checked)
  {
  if (checked == true)
    WRITE ("<INPUT TYPE=CHECKBOX NAME=\"%s\" VALUE=\"%s\" CHECKED>\n", name, value);
  else
    WRITE ("<INPUT TYPE=CHECKBOX NAME=\"%s\" VALUE=\"%s\">\n", name, value);    
  }      
  
void HTML::JSFormCheckbox (char *name, char *value, char *onclick, bool checked)
  {
  if (checked == true)
    WRITE ("<INPUT TYPE=CHECKBOX NAME=\"%s\" onClick=\"%s\" VALUE=\"%s\" CHECKED>\n", name, onclick, value);
  else
    WRITE ("<INPUT TYPE=CHECKBOX NAME=\"%s\" onClick=\"%s\" VALUE=\"%s\">\n", name, onclick, value);    
  }    
  
void HTML::FormImage (char *name, char *src)
  {
  WRITE ("<INPUT TYPE=IMAGE NAME=\"%s\" SRC=\"%s\">\n", name, src);
  }  
  
void HTML::FormButton (char *name, char *value)
  {
  WRITE ("<INPUT TYPE=SUBMIT NAME=\"%s\" VALUE=\"%s\">\n", name, value);
  }  
        
void HTML::FormReset (char *value)
  {
  WRITE ("<INPUT TYPE=RESET VALUE=\"%s\">\n", value);
  }
  
void HTML::soErrorPage (char *error)
  {                    
  fprintf (stdout, "Content-type: text/html\n\n");     
  fprintf (stdout, "<p><h1><FONT color=\"#FF0000\">Error:</font> %s</h1><p>\n", error);
  }  

void HTML::dumpSL (StringList *SL)
  {
  Head ("text/plain", "", "");
  for (int i=0; i< SL->Count(); ++i)
    {
    WRITE ((char *)SL->getString(i).cstr());
    }
  }

void HTML::dumpPChar (char *pc)
  {
  Head ("text/plain", "", "");
  if (pc == NULL) pcWRITE ("NULL"); 
  else pcWRITE (pc);
  }                  
  
void HTML::LineaLanguage ()
  {
  TBuffer Buf;
  Linea ();
  PrintString (L->get(L_CHOOSELANGUAGE));
  sprintf (Buf, "%s?lang=eng", PATH_POSTMAN); URL ("English", Buf);
  sprintf (Buf, "%s?lang=spa", PATH_POSTMAN); URL ("Espa&ntilde;ol", Buf);
  sprintf (Buf, "%s?lang=val", PATH_POSTMAN); URL ("Valenci&agrave;", Buf);
  }
  
void HTML::PrintStatusLine (const char *sl, const char *user, const char *imapserver)
  {
  if (FileExists (FILE_FOR_NO_ACCEPT_NEW_CONNS) == true)
    {
    TBuffer amsg;
    GetBufferFromFile (FILE_FOR_NO_ACCEPT_NEW_CONNS, amsg, CMAXBUFFER);
    WRITE ("<TABLE width=\"100%%\"><TR><TD align=left><B><FONT color=\"#FF0000\"><BLINK>%s</BLINK></FONT></B></TD><TD align=right><B><FONT size=2 color=\"%s\">%s: %s (%s)</FONT></B></TD></TR></TABLE>\n",
            amsg, BLUE, L->get(L_USER), user, imapserver);      
    }
  else
    {
    WRITE ("<TABLE width=\"100%%\"><TR><TD align=left><B>%s</B></TD><TD align=right><B><FONT color=\"%s\">%s: %s (%s)</FONT></B></TD></TR></TABLE>\n",
           sl, BLUE, L->get(L_USER), user, imapserver);
    }
  }
  
void HTML::BotoneraDisplayMsg (char *lang, char *user, char *connid, UserOptions *uo, bool isdeleted, ULongList *LL)
  {
  long actualpos, totalmsgs, trueactualpos, truenummsg;
  int validator, pageid, actualcmd;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId();
  actualcmd = CMD_MG_DISPLAY;
  trueactualpos = LL->indexOf (actualpos);
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(CMD_MG_DISPLAY, lang, connid, true));
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();        
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/mg_display/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();      
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"),  uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON PREV MSG
      if (trueactualpos > 0)
        {
        truenummsg = LL->elementAt((int)(trueactualpos - 1));
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVMSG), L->get(ALT_PREVMSG));
        WRITE (" %s\n", L->get(ALT_PREVMSG));
        WRITE ("</A>");
        EndTD();
        }
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVMSG_INA), L->get(ALT_PREVMSG_INA));
        WRITE (" %s\n", L->get(ALT_PREVMSG_INA));
        EndTD();
        }
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON NEXT MSG
      if (trueactualpos < totalmsgs - 1)
        {
        truenummsg = LL->elementAt((int)(trueactualpos + 1));
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTMSG), L->get(ALT_NEXTMSG));
        WRITE (" %s\n", L->get(ALT_NEXTMSG));
        WRITE ("</A>");
        EndTD();
        }
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTMSG_INA), L->get(L_NEXTMSG_INA));
        WRITE (" %s\n", L->get(ALT_NEXTMSG_INA));
        EndTD();
        }
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON REPLY
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_reply/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_REPLYMSG), L->get(ALT_REPLYMSG));
      WRITE (" %s\n", L->get(ALT_REPLYMSG));
      WRITE ("</A>");                                                      
      EndTD();        
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON FORWARD
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm_forward/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FORWARDMSG), L->get(ALT_FORWARDMSG));
      WRITE (" %s\n", L->get(ALT_FORWARDMSG));
      WRITE ("</A>");
      EndTD();              
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON SAVE ADDRESS
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/ab/ab_takeaddress/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_SAVEADDRESS), L->get(ALT_SAVEADDRESS));
      WRITE (" %s\n", L->get(ALT_SAVEADDRESS));
      WRITE ("</A>");
      EndTD();              
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON DUMPMSG
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mg_dump/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd, actualpos);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_DUMPMSG), L->get(ALT_DUMPMSG));
      WRITE (" %s\n", L->get(ALT_DUMPMSG));
      WRITE ("</A>");
      EndTD();  
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON DELNEXTMSG
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (trueactualpos < totalmsgs - 1) {truenummsg = LL->elementAt((int)(trueactualpos + 1));} else {truenummsg = LL->elementAt((int)(trueactualpos));}
      if (isdeleted == false)
        {
        WRITE ("<A HREF=\"%s/mg_deldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_DELNEXTMSG), L->get(ALT_DELNEXTMSG));
        WRITE (" %s\n", L->get(ALT_DELNEXTMSG));
        WRITE ("</A>");                  
        }
      else
        {
        WRITE ("<A HREF=\"%s/mg_undeldisplay/noop/%s/%s/%s/%d/%d/%d/%ld/\">",PATH_POSTMAN, lang, user, connid, validator, pageid, 
               actualcmd, truenummsg);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_UNDELNEXTMSG), L->get(ALT_UNDELNEXTMSG));
        WRITE (" %s\n", L->get(ALT_UNDELNEXTMSG));
        WRITE ("</A>");                          
        }
      EndTD();
      /**/BeginTD (10, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
    EndTR();
  EndTable (); 
  PrintStatusLine (uo->getStatusLine(), user, uo->getImapServer()); 
  } 
  
void HTML::BotoneraMailboxIndex (char *lang, char *user, char *connid, UserOptions *uo)
  {
  long newbegin, actualpos, totalmsgs, aMAXMSGSFORPAGE;
  int validator, actualcmd;                           
  
  actualpos = uo->getBeginPage();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  actualcmd = CMD_MB_INDEX;
  
  aMAXMSGSFORPAGE = MAXMSGSFORPAGE;
  if (aMAXMSGSFORPAGE <= 0) aMAXMSGSFORPAGE = totalmsgs;
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/indexmailbox/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, 
             connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON PREVIOUS PAGE
      if (actualpos > 1)  
        {
        newbegin = actualpos - aMAXMSGSFORPAGE; if (newbegin <= 0) newbegin = 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, 
               validator, uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVPAGE_ACT), L->get(ALT_PREVPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_PREVPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }
      else 
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_PREVPAGE_INA), L->get(ALT_PREVPAGE_INA));
        WRITE (" %s\n", L->get(ALT_PREVPAGE_INA));
        EndTD();
        }
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON NEXT PAGE
      if (actualpos + aMAXMSGSFORPAGE <= totalmsgs)
        {
        newbegin = actualpos + aMAXMSGSFORPAGE; 
        if (newbegin + aMAXMSGSFORPAGE > totalmsgs)
          {
          ldiv_t ld;
          ld = ldiv (totalmsgs, uo->getMsgsPerIndexPage());
          newbegin = (ld.quot * uo->getMsgsPerIndexPage()) + 1;
          }
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTPAGE_ACT), L->get(ALT_NEXTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_NEXTPAGE_ACT));
        WRITE ("</A>");
        EndTD(); 
        }
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_NEXTPAGE_INA), L->get(ALT_NEXTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_NEXTPAGE_INA));
        EndTD();
        }  
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON FIRST PAGE
      if (actualpos > 1)
        {
        newbegin = 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FIRSTPAGE_ACT), L->get(ALT_FIRSTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_FIRSTPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }          
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_FIRSTPAGE_INA), L->get(ALT_FIRSTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_FIRSTPAGE_INA));
        EndTD();
        }  
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON LAST PAGE
      if (actualpos + aMAXMSGSFORPAGE <= totalmsgs)
        {
        ldiv_t ld;
        ld = ldiv (totalmsgs, uo->getMsgsPerIndexPage());
        newbegin = (ld.quot * uo->getMsgsPerIndexPage()) + 1;
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, lang, user, connid, validator, 
               uo->getPageId(), actualcmd, newbegin);
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LASTPAGE_ACT), L->get(ALT_LASTPAGE_ACT));
        WRITE (" %s\n", L->get(ALT_LASTPAGE_ACT));
        WRITE ("</A>");
        EndTD();
        }          
      else
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LASTPAGE_INA), L->get(ALT_LASTPAGE_INA));
        WRITE (" %s\n", L->get(ALT_LASTPAGE_INA));
        EndTD();
        }
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON CM
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/cm/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_COMPOSEMSG), L->get(ALT_COMPOSEMSG));
      WRITE (" %s\n", L->get(ALT_COMPOSEMSG));
      WRITE ("</A>");
      EndTD();                
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON MB
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/mb_show/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_MAILBOXES), L->get(ALT_MAILBOXES));
      WRITE (" %s\n", L->get(ALT_MAILBOXES));
      WRITE ("</A>");
      EndTD();                      
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON OP
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/op/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_SHOWOPTIONS), L->get(ALT_SHOWOPTIONS));
      WRITE (" %s\n", L->get(ALT_SHOWOPTIONS));
      WRITE ("</A>");
      EndTD();                      
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON AB  
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/ab/ab_showall/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_ADDBOOK), L->get(ALT_ADDBOOK));
      WRITE (" %s\n", L->get(ALT_ADDBOOK));
      WRITE ("</A>");
      EndTD();                      
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON MB_EXPUNGE   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmPurge() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/mb_expunge/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/mb_expunge/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_EXPUNGE), L->get(ALT_EXPUNGE));
      WRITE (" %s\n", L->get(ALT_EXPUNGE));
      WRITE ("</A>");
      EndTD();        
      /**/BeginTD (8, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();   
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, uo->getPageId(), actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
    EndTR();

  EndTable ();
  PrintStatusLine (uo->getStatusLine(), user, uo->getImapServer()); 
  }   
  
void HTML::BotoneraComun (int cmdactual, char *lang, char *user, char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK       
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(cmdactual, lang, connid, true));
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();   
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      switch (cmdactual)
        {
        case CMD_MB_SHOW: WRITE ("<A HREF=\"%s/help/mb_show/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_CM: WRITE ("<A HREF=\"%s/help/cm/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_OP: WRITE ("<A HREF=\"%s/help/op/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB: case CMD_AB_SHOWALL: case CMD_AB_SORT: case CMD_AB_FROM_CM:
        case CMD_AB_ADDTO: case CMD_AB_ADDCC: case CMD_AB_ADDBCC:
          WRITE ("<A HREF=\"%s/help/ab/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB_NEWENTRY:  WRITE ("<A HREF=\"%s/help/ab_newentry/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        case CMD_AB_EDITENTRY: WRITE ("<A HREF=\"%s/help/ab_editentry/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); break;
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE
      EmptyCellx2();      
      //BUTTON NEW ENTRY IN ADDRESSBOOK
      if ((cmdactual == CMD_AB_SHOWALL) || (cmdactual == CMD_AB_SORT))
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/ab/ab_newentry/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); 
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_AB_NEWENTRY), L->get(ALT_AB_NEWENTRY));
        WRITE (" %s\n", L->get(ALT_AB_NEWENTRY));
        WRITE ("</A>");
        EndTD();          
        EmptyCellx2();
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/ab/ab_dump/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); 
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_AB_DUMP), L->get(ALT_AB_DUMP));
        WRITE (" %s\n", L->get(L_AB_DUMP));
        WRITE ("</A>");
        EndTD();          
        EmptyCellx2();              
        }
      //BUTTON ABOUT
      if (cmdactual == CMD_OP)
        {
        BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
        WRITE ("<A HREF=\"%s/help/about/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual); 
        WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_ABOUT), L->get(ALT_ABOUT));
        WRITE (" %s\n", L->get(ALT_ABOUT));
        WRITE ("</A>");
        EndTD();          
        EmptyCellx2();
        }
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, cmdactual);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD(); 
    EndTR();
  EndTable ();  
  PrintStatusLine (uo->getStatusLine(), user, uo->getImapServer()); 
  }

void HTML::BotoneraComposeMsg (int thecmd, char *lang, char *user, char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(thecmd, lang, connid, true));      
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();                                                           
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/cm/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd); 
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE 
      EmptyCellx2();
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, thecmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();
    EndTR();
  EndTable ();  
  PrintStatusLine (uo->getStatusLine(), user, uo->getImapServer()); 
  }

void HTML::BotoneraAttachs (char *lang, char *user, char *connid, UserOptions *uo)
  {
  long actualpos, totalmsgs;
  int validator, pageid, actualcmd;
  
  actualpos = uo->getCurrentPosition();
  totalmsgs = uo->getTotalMsgs();
  validator = uo->getValidator();
  pageid    = uo->getPageId ();
  actualcmd = CMD_AT_SHOW;
  
  BeginTable (0, 0, 0, 100);
    BeginTR(uo->getSkinAtribProp("COLOR_BGBOTONERA"));
      //BUTTON BACK
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("%s", uo->getLastStringCmd(CMD_AT_SHOW, lang, connid, true));      
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(ALT_BACK));
      WRITE (" %s\n", L->get(ALT_BACK));
      WRITE ("</A>");
      EndTD();        
      //SPACE 
      EmptyCellx2();
      //BUTTON HELP   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      WRITE ("<A HREF=\"%s/help/at_show/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd); 
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_HELP), L->get(ALT_HELP));
      WRITE (" %s\n", L->get(ALT_HELP));
      WRITE ("</A>");
      EndTD();  
      //SPACE 
      EmptyCellx2();
      //BUTTON LOGOUT   
      BeginTD (1, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center");
      if (uo->getConfirmLogout() == true)    
        {
        WRITE ("<A HREF=\"%s/conf_act/logout/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      else
        {  
        WRITE ("<A HREF=\"%s/logout/noop/%s/%s/%s/%d/%d/%d/\">", PATH_POSTMAN, lang, user, connid, validator, pageid, actualcmd);
        }
      WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_LOGOUT), L->get(ALT_LOGOUT));
      WRITE (" %s\n", L->get(ALT_LOGOUT));
      WRITE ("</A>");
      EndTD();
      BeginTD (100, uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_BOTONERA_SIZE"), "center"); WRITE (HTML_HSP); EndTD();
    EndTR();
  EndTable ();  
  PrintStatusLine (uo->getStatusLine(), user, uo->getImapServer()); 
  }

void HTML::PrintIndexPage (char *lang, char *mailbox, char *user, char *connid, 
                           StringList *SL, StringList *SL2,
                           UserOptions *uo)
  {
  TBuffer pag_to_actualizar;
  TSBuffer sbuf;
  long nummsg, truenummsg, beginshow, endshow;
  uint i, cont;
  char *subject, *from;
  ldiv_t ld;
  long newbegin, aMAXMSGSFORPAGE = MAXMSGSFORPAGE;
  if (aMAXMSGSFORPAGE <= 0) aMAXMSGSFORPAGE = uo->getTotalMsgs();

  Head ("text/html", "", "");
  HTMLBegin ();
  
  ld = ldiv (uo->getTotalMsgs(), aMAXMSGSFORPAGE);
  newbegin = (ld.quot * aMAXMSGSFORPAGE) + 1;
  sprintf (pag_to_actualizar, "%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, newbegin);
  
  if (strcasecmp (mailbox, "INBOX") == 0)
    {
    Title (L->get(TIT_INDEXMAILBOX), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), 
           uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), uo->getRefreshTime(), pag_to_actualizar);  
    }
  else
    {
    Title (L->get(TIT_INDEXMAILBOX), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),           
           uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
    }
  BotoneraMailboxIndex (lang, user, connid, uo);
  nummsg = uo->getBeginPage();
  if (uo->get_use_javascript() == true)
    {
    JavaScript("function putOn()\n{\nfor (i = 0; i < document.forms[0].elements.length; i++)\n{\ndocument.forms[0].elements[i].checked = true;\n}\n}\n\nfunction putOff()\n{\nfor (i = 0; i < document.forms[0].elements.length; i++)\n{\ndocument.forms[0].elements[i].checked = false;\n}\n}\n");
    }
  i = 0;
  WRITE ("<CENTER><TABLE width=\"100%%\"><TR><TD>\n");  /****begin_silly_table****/
  BeginFormPost (PATH_POSTMAN);
  BeginTable (0, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLEMSGINDEX"));
  BeginTDColSpanCenter (7); 
  if (CALCULATEMAILBOXSIZE == 1)
    {
    WRITE (L->get(L_INDEXMAILBOX3), mailbox, uo->getTotalMsgs(), FormatCountBytes1Dec(uo->getMailboxSize())); 
    }
  else
    {
    WRITE (L->get(L_INDEXMAILBOX), mailbox, uo->getTotalMsgs());
    }
  beginshow =  nummsg;  
  endshow   = nummsg - 1 + (SL->Count() / 7);
  if (endshow == 0) {beginshow = 0;}
  if (endshow > 0) {WRITE (L->get(L_MSGS_SHOWN), beginshow, endshow);}
  EndTD(); //WRITE ("</TD>");
  EndTR(); 
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  if (uo->get_use_javascript() == true)
    {
    BeginTD (uo->getSkinAtribProp("COLOR_BGFILASMSGINDEX")); JSFormCheckbox ("jscheckall", "", "if (this.checked) {putOn()} else {putOff()}", false); EndTD();     
    }
  else
    {
    BeginTD(true,  1, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); WRITE (HTML_HSP); EndTD(); 
    }
  BeginTD(true,  5, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/number/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_NUMBER));}
  else
    {WRITE ("%s", L->get(L_NUMBER));}
  EndTD();     
  BeginTD(true,  5, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
    WRITE ("%s", L->get(L_FLAGS)); 
  EndTD(); 
  BeginTD(true, 10, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/date/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_DATE));}
  else
    {WRITE ("%s", L->get(L_DATE));}
  EndTD();     
  BeginTD(true, 25, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/from/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_FROM));}
  else
    {WRITE ("%s", L->get(L_FROM));}
  EndTD();     
  BeginTD(true, 49, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/subject/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_SUBJECT));}
  else
    {WRITE ("%s", L->get(L_SUBJECT));}
  EndTD();     
  BeginTD(true,  5, 0, uo->getSkinAtribProp("FONT_MAIN"), 2, "center"); 
  if (uo->getTotalMsgs() > 0)
    {WRITE ("<A HREF=\"%s/mb_sort/size/%s/%s/%s/%d/%d/%d/\">%s</A>", PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, L->get(L_SIZE));}
  else
    {WRITE ("%s", L->get(L_SIZE));}
  EndTD();     
  EndTR();
  
  cont = SL->Count();
  while (i < cont) 
    { 
    truenummsg = xatoldef (SL->getString(i).cstr(), 1);
    
    //CHECKBOX
    WRITE ("<TR valign=baseline bgcolor=\"%s\">", uo->getSkinAtribProp("COLOR_BGFILASMSGINDEX"));
    BeginTD();
    FormCheckbox ("msgnumber", xltoa(truenummsg), false); EndTD();

    //1. POSITION
    WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%s.</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), SL->getString(i).cstr());
    ++i;
    
    //2. FLAGS
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), DisplayFlags((const char *)SL->getString(i).cstr(), false, "", 0));
    ++i;
    
    //3. DATE
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), (char *)SL->getString(i).cstr());
    ++i;

    //4. FROM
    from = HTML2text((char *)SL->getString((uint)i).cstr());
    if (IsEmpty(from) == true) 
      {
      from = xstrdup (L->get(L_NOFROM));
      }
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), from);
    free (from);
    ++i;

    //5. SUBJECT
    subject = HTML2text((char *)SL->getString((uint)i).cstr());
    if (IsEmpty(subject) == true) {subject = xstrdup (L->get(L_NOSUBJECT));}
    WRITE ("<TD nowrap><FONT size=2 face=\"%s\"><B><A HREF=\"%s/mg_display/noop/%s/%s/%s/%d/%d/%d/%ld/\">%s</A></B></FONT></td>\n", 
            uo->getSkinAtribProp("FONT_TABLE"), PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MB_INDEX, 
            truenummsg, subject);
    free (subject);
    ++i;
    
    //6. SIZE   
    strncpy (sbuf, (char *)SL->getString(i).cstr(), CSMALLBUFFER);                                                                                                           
    if (sbuf[strlen(sbuf) - 1] != 'K')
      {
      WRITE ("<TD align=\"right\"><FONT color=\"%s\" size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", RED, uo->getSkinAtribProp("FONT_TABLE"), sbuf);
      }
    else
      {
      WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%s</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), sbuf);
      }
    ++i;
    
    //7. UID
    //WRITE ("<TD align=\"right\"><FONT size=2 face=\"%s\"><B>%ld</B></FONT></TD>\n", uo->getSkinAtribProp("FONT_TABLE"), SL->getString(i).cstr());
    ++i;
    
    EndTR();

    ++nummsg;
    }
  EndTable();
    
  if (endshow < uo->getTotalMsgs()) //PARA PASAR PAGINA AL FINAL DEL INDEX
    {   
    newbegin = uo->getBeginPage() + aMAXMSGSFORPAGE; 
    if (newbegin + aMAXMSGSFORPAGE > uo->getTotalMsgs())
      {
      ld = ldiv (uo->getTotalMsgs(), uo->getMsgsPerIndexPage());
      newbegin = (ld.quot * uo->getMsgsPerIndexPage()) + 1;
      } 
    WRITE ("<CENTER><FONT size=2><A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">[%s]</A></FONT></CENTER>", PATH_POSTMAN, lang, user, connid, uo->getValidator(),
            uo->getPageId(), CMD_MB_INDEX, newbegin,  L->get(L_NEXTPAGE_ACT));
    }
  else //ACTUALIZAR BUZON
    {
    WRITE ("<CENTER><FONT size=2><A HREF=\"%s\">[%s]</A></FONT></CENTER>", pag_to_actualizar, L->get(L_REFRESH));
    }  
  
  BeginTable (5, 0, 0, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  WRITE ("<TD><FONT>"); EndTD();
  
  BeginTDCenter();
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_INDEX);
  FormButton ("theaction", L->get(L_MARK));
    
  Label (L->get(L_MSGS_AS));
  FormBeginSelect ("cmd"); 
  FormAddOptionSelect (true,  "fg_delete",     L->get(L_DELETEMSG));
  FormAddOptionSelect (false, "fg_undelete",   L->get(L_UNDELETEMSG));
  FormAddOptionSelect (false, "fg_answered",   L->get(L_ANSWEREDMSG));
  FormAddOptionSelect (false, "fg_unanswered", L->get(L_UNANSWEREDMSG));
  FormAddOptionSelect (false, "fg_unseen",     L->get(L_UNSEENMSG));
  FormAddOptionSelect (false, "fg_seen",       L->get(L_SEENMSG));  
  FormAddOptionSelect (false, "fg_flagged",    L->get(L_FLAGGEDMSG));
  FormAddOptionSelect (false, "fg_unflagged",  L->get(L_UNFLAGGEDMSG));
  FormEndSelect();
  EndTD();
  
  BeginTD();  WRITE (HTML_HSP); EndTD();
  
  BeginTDCenter();
  FormBeginSelect ("cmd2"); 
  FormAddOptionSelect (false, "mg_move",     L->get(L_MOVEMSG));
  FormAddOptionSelect (false, "mg_copy",     L->get(L_COPYMSG));
  FormAddOptionSelect (true,  "mb_change",   L->get(L_CHANGEMAILBOX));
  FormEndSelect();
  FormBeginSelect ("mailboxname"); 
  FormAddOptionSelect (true, INBOX, "INBOX");
  TSBuffer abuzon;
  for (int Y=0; Y<SL2->Count(); ++Y)
    {
    strncpy (abuzon, SL2->getString(Y).cstr() + strlen(HTML_DCOMILLAS), CSMALLBUFFER);
    abuzon [strlen(abuzon) - strlen(HTML_DCOMILLAS)] = '\0';
    FormAddOptionSelect (false, SL2->getString(Y).cstr(), abuzon);
    }
  FormEndSelect(); 
  FormButton ("theaction2", L->get(L_EXECUTE));
  EndTD();
  EndTR();
  
  EndForm ();
  
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("cmd", "mb_search");      
  FormHidden ("lastcmd", CMD_MB_INDEX);    
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  WRITE ("<TD><FONT>"); EndTD();
  BeginTDColSpanCenter(3);   
  FormButton ("fd_tosearch", L->get(L_SEARCH));  
  FormBeginSelect ("fieldtosearch"); 
  FormAddOptionSelect (true,  "fd_subject", L->get(L_SEARCH_SUBJECT));
  FormAddOptionSelect (false, "fd_from",    L->get(L_SEARCH_FROM));
  FormAddOptionSelect (false, "fd_body",    L->get(L_SEARCH_BODY));
  FormAddOptionSelect (false, "fd_all",     L->get(L_SEARCH_ALL));
  FormEndSelect();
  WRITE ("<INPUT TYPE=TEXT NAME=\"whatsearch\" SIZE=\"40\" MAXLENGTH=\"100\" VALUE=\"\">\n"); 
  EndTD();
  EndTR();    
  EndForm();
  
  EndTable();
  WRITE("</TD></TR></TABLE></CENTER>\n");      /****end_silly_table****/
  if (DEBUGGING_POSTMAN == 1) 
    {
    WRITE ("<BR><A HREF=\"%s/debug/noop/%s/%s/%s/%d/%d/%d/%ld/\">PAGINA DE DEBUG</A>\n", PATH_POSTMAN, lang, user, connid, uo->getValidator(),uo->getPageId(), CMD_MB_INDEX, uo->getCurrentPosition());
    }
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::dumpAttach (char *chunk, xulong len, StringList *SLAttachInfo)
  {
  TBuffer tiposubtipo, fn;
  strncpy (tiposubtipo, SLAttachInfo->getString(3).cstr(), CMAXBUFFER);
  strncat (tiposubtipo, "/", 1);
  strncat (tiposubtipo, SLAttachInfo->getString(4).cstr(), CMAXBUFFER);
  if (SLAttachInfo->getString(6).cstr() != NULL) strncpy (fn, SLAttachInfo->getString(6).cstr(), CMAXBUFFER);
  else if (SLAttachInfo->getString(7).cstr() != NULL) strncpy (fn, SLAttachInfo->getString(7).cstr(), CMAXBUFFER); 
  else initStr (fn);
  HeadInline (tiposubtipo, fn, len);  
  if (chunk == NULL) pcWRITE ("NULL"); 
  else pcWRITEBuffer (chunk, len);
  }  
  
void HTML::dumpRawAttachPC (char *pcheader, xulong lenheader, char *pcbody, xulong lenbody)
  {
  pcWRITEBuffer (pcheader, lenheader);
  pcWRITEBuffer (pcbody, lenbody);
  }    
  
void HTML::dumpRawAttachSL (StringList *SLAttachHeader, char *pcbody, xulong lenbody)
  {
  for (int i = 0; i < SLAttachHeader->Count() - 1; ++i)
    {
    pcWRITEBuffer ((char *)SLAttachHeader->getString((uint)i).cstr(), 
                   strlen((char *)SLAttachHeader->getString((uint)i).cstr()));
    }
  pcWRITEBuffer ("\r\n", 2);  
  pcWRITEBuffer (pcbody, lenbody);  
  }      
  
void HTML::dumpAttachWithoutHead (char *chunk, xulong len)
  {
  if (chunk == NULL) pcWRITE ("NULL"); 
  else pcWRITEBuffer (chunk, len);
  }    
  
void HTML::PrintMessageHeader 
                        (char *lang, char *user, char *connid, 
                         long nummsgs, 
                         char *aflags, char *ato, char *afrom, char *asubject, char *adate, char *asize,
                         StringList *SLAttachs,
                         UserOptions *uo, ULongList *LL)
  {
  bool isdeleted;
  XString JS;
  xulong i, Partes, size;
  TBuffer attid, contenttype, basura, molde;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_MESSAGEHEADER), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  if (strchr (aflags,  FDELETED) == NULL) isdeleted = false; else isdeleted = true;
  BotoneraDisplayMsg (lang, user, connid, uo, isdeleted, LL);
  BeginTable (0, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (3); WRITE (L->get(L_MESSAGE), uo->getCurrentPosition(), nummsgs, asize); BR(); EndTD();
  EndTR();   
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_FLAGS)); EndTD();    
  sprintf (molde, "%s/%s/%s/%d/%d/%d/%ld/\">", 
           lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY, uo->getCurrentPosition());
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 1); JS = XString (DisplayFlags((const char *)aflags, true, molde, 1)); WRITE ((char *)JS.cstr()); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right", 1);JS = XString (DisplayFlags((const char *)aflags, true, molde, 2)); WRITE ((char *)JS.cstr()); EndTD(); 
  EndTR();       
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_SUBJECT)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = XString (HTML2text(asubject)); WRITE ((char *)JS.cstr()); EndTD();
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_TO)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = XString (HTML2text(ato)); WRITE ((char *)JS.cstr()); EndTD();                                                 
  EndTR();  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_FROM)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = XString (HTML2text(afrom)); WRITE ((char *)JS.cstr()); EndTD();
  EndTR();  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(10); Label (L->get(L_DATE)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "left", 2); JS = XString (HTML2text(adate)); WRITE ((char *)JS.cstr()); EndTD();
  EndTR();  
  EndTable();
  Linea ();
  Partes = SLAttachs->Count();
  if (Partes > 1) 
    {
    BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));  
    H4(L->get (L_STRUCTMIME));
    for (i=0; i< Partes; ++i)
      {
      sscanf ((char *)SLAttachs->getString((xuint)i).cstr(), "%s %s %ld %s", attid, contenttype, &size, basura);
      WRITE ("<A HREF=\"%s/at_dump/noop/%s/%s/%s/%d/%d/%d/%ld/%s/\">%s</A>", 
             PATH_POSTMAN, lang, user, connid, uo->getValidator(), uo->getPageId(), CMD_MG_DISPLAY,
             uo->getCurrentPosition(), attid, SLAttachs->getString((uint)i).cstr());
      if ((strstr (contenttype, "TEXT/") != NULL) && (size <= MAXTEXTSIZEATTACHFORDISPLAY))
        {
        WRITE ("<FONT COLOR=\"%s\">  %s</FONT>", RED, L->get(L_ATTACHSHOWED));
        }       
      WRITE ("<BR>\n");       
      }
    Linea ();
    EndFont();
    }
  }  

void HTML::PrintMailboxesPage (char *lang, char *mailbox, char *user, char *connid, 
                               StringList *SLMailboxes, UserOptions *uo)
  {
  int i;
  StringList *SLMailboxesSinComillas;
  TSBuffer abuzon;
  SLMailboxesSinComillas = new StringList();
  for (i=0; i<SLMailboxes->Count(); ++i)
    {
    strncpy (abuzon, SLMailboxes->getString(i).cstr() + strlen(HTML_DCOMILLAS), CSMALLBUFFER);
    abuzon [strlen(abuzon) - strlen(HTML_DCOMILLAS)] = '\0';
    SLMailboxesSinComillas->Add (abuzon);
    }
  
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_MAILBOXES), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  BotoneraComun (CMD_MB_SHOW, lang, user, connid, uo);
  BeginTable (1, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpanCenter (2); WRITE (L->get(L_INDEXMAILBOX4), mailbox, uo->getTotalMsgs()); EndTD();
  BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
  BeginFormPost (PATH_POSTMAN);
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); FormButton ("theaction", L->get(L_CHANGEMAILBOX)); EndTD();   
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_change");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);  
  FormBeginSelect ("mailboxname"); 
  FormAddOptionSelect (true, INBOX, "INBOX");
  for (i=0; i<SLMailboxes->Count(); ++i)
    {
    FormAddOptionSelect (false, SLMailboxes->getString(i).cstr(), SLMailboxesSinComillas->getString(i).cstr());
    }
  FormEndSelect();  
  EndTD();
  EndTR(); 
  EndForm ();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_create");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);
  BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"), "right"); FormButton ("theaction", L->get(L_CREATEMAILBOX)); EndTD();     
  BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE")); FormEdit (16, 16, "mailboxname", "");     EndTD();
  EndForm ();
  EndTR();
  
  if (SLMailboxes->Count () > 0)
    {
    BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
    BeginFormPost (PATH_POSTMAN);
    FormHidden ("lang", lang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("validator", uo->getValidator());
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_MB_SHOW);
    if (CONFIRM_DELETEMAILBOX == 1)
      {
      FormHidden ("subcmd", "mb_delete");
      FormHidden ("cmd", "conf_act");
      }
    else FormHidden ("cmd", "mb_delete");
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); FormButton ("theaction", L->get(L_DELETEMAILBOX)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
    FormBeginSelect ("mailboxname"); 
    for (int j=0; j<SLMailboxes->Count(); ++j)
      {
      if (j == 0)  FormAddOptionSelect (true, SLMailboxes->getString(j).cstr(), SLMailboxesSinComillas->getString(j).cstr());
      else FormAddOptionSelect (false, SLMailboxes->getString(j).cstr(), SLMailboxesSinComillas->getString(j).cstr());
      }
    FormEndSelect();  
    EndTD();
    EndTR();   
    EndForm ();          
    BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
    BeginFormPost (PATH_POSTMAN);
    FormHidden ("lang", lang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("cmd", "mb_rename");
    FormHidden ("validator", uo->getValidator());
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_MB_SHOW);
    BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"), "right"); FormButton ("theaction", L->get(L_RENAMEMAILBOX)); EndTD();
    BeginTD(uo->getSkinAtribProp("COLOR_BGTABLE"));
    FormBeginSelect ("oldmailboxname"); 
    for (int k=0; k<SLMailboxes->Count(); ++k)
      {
      if (k == 0)  FormAddOptionSelect (true, SLMailboxes->getString(k).cstr(), SLMailboxesSinComillas->getString(k).cstr());
      else FormAddOptionSelect (false, SLMailboxes->getString(k).cstr(), SLMailboxesSinComillas->getString(k).cstr());    
      }
    FormEndSelect();  
    WRITE ("%s<B>%s</B>%s\n", HTML_HSP, L->get(L_LEX_TO), HTML_HSP);
    FormEdit (16, 16, "newmailboxname", ""); 
    EndTD();
    EndTR();
    EndForm ();
    }  
  
  BeginTR(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"));
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "mb_dump");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_MB_SHOW);   
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG"), "right"); FormButton ("theaction", L->get(L_DOWNLOADMAILBOX)); EndTD();     
  BeginTD(); WRITE(HTML_HSP); WRITE(HTML_HSP); Label (mailbox); EndTD();
  EndForm ();
  EndTable();          
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  delete SLMailboxesSinComillas;
  }  
        
void HTML::PrintConfirmPage (char *stcmd, char *stlang, char *mailbox, 
                             char *user, char *connid, int validator, int pageid, UserOptions *uo)
  {
  int cmd;
  cmd = TranslateCmd (stcmd);
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_CONFIRMACTION), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);  
  Linea ();
  BeginCenter(); 
  switch (cmd)
    {
    case CMD_MB_DELETE:  RedLabel (L->get(MSG_CONF_DELETEMAILBOX)); break;
    case CMD_LOGOUT:     RedLabel (L->get(MSG_CONF_LOGOUT)); break;
    case CMD_MB_EXPUNGE: RedLabel (L->get(MSG_CONF_EXPUNGE)); break;
    }
  EndCenter ();
  BeginCenter();
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", stcmd);
  FormHidden ("validator", validator);
  FormHidden ("pageid", pageid);
  FormHidden ("lastcmd", CMD_CONF_ACT);
  if (cmd == CMD_MB_DELETE) {FormHiddenQuoted ("mailboxname", mailbox);}  
  FormButton ("theaction", L->get(L_CONFIRM));    
  EndForm ();  
  //EndTD();
  //EndTR();
  //EndTable ();
  EndCenter ();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
   
void HTML::PrintComposeMsgPage (int thecmd, int subcmd, char *lang, char *user, char *connid,     
                                char *to, char *cc, char *bcc, char *subject, 
                                char *replyto, char *from, char *date,
                                char *firsttext,
                                UserOptions *uo)
                                                              
  {
  TSBuffer abuf;
  TBuffer amsg;
  int thecmd_x, p;
  XString JS, JS2, JS3, JS4;
  Head ("text/html", "", "");
  HTMLBegin ();
  
  if ((uo->get_use_javascript() == true) && (USETIMEOUT == 1))
    {
    long restotime = (uo->getTimeout() - (5 * 60)) * 1000;
    sprintf (amsg, "window.setTimeout(\"AlertToDisabled();\",%ld);function AlertToDisabled(){alert(\"%s\")}", restotime, L->get(MSG_GO_TO_DISABLED));
    JavaScript(amsg);
    }
  
  switch (thecmd)
    {
    case CMD_CM: Title (L->get(TIT_COMPOSEMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    case CMD_CM_REPLY:   Title (L->get(TIT_REPLYMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    case CMD_CM_FORWARD: Title (L->get(TIT_FORWARDMSG), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"), uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL); break;
    }
  BotoneraComposeMsg (subcmd, lang, user, connid, uo);
  /*silly_table*/WRITE ("<CENTER><TABLE width=\"100%%\"><TR><TD>\n");
  BeginFormPost (PATH_POSTMAN);
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));
  FormButton ("cm_save", L->get(L_SAVE));
  //REMOVED AT THIS MOMENT FormButton ("cm_cancel",   L->get(L_CANCELMSG));
  FormButton ("cm_send",   L->get(L_SENDMSG));
  FormButton ("at_show", L->get(L_ATTACHMSG));
  FormButton ("cm_cleanall", L->get(L_CLEANALL));
  FormButton ("ab_from_cm", L->get(L_ADDBOOK));  
  FormHidden ("lang", lang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", thecmd);  
  FormHidden ("cmd", "cm");
  FormHidden ("validator", uo->getValidator());
  FormHidden ("numattachs", uo->numAttachs());  
  
  //ESTO ES PARA QUE AL VOLVER A UN MENSAJE QUE ESTOY HACIENDO UN REPLY (o FORWARD) NO ME EJECUTE
  //   EL REPLY SINO QUE BUSQUE EL MSG GUARDADO Y LO MUESTRE
  thecmd_x = thecmd;
  if (uo->getLastCmd() != CMD_MG_DISPLAY) thecmd_x = CMD_CM;
  switch (thecmd_x)
    {
    case CMD_CM: 
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, 50, "to", HTML2text(getLineFile(uo->getfnSavedMsg(), 2, 2)));           
      EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, 50, "cc", HTML2text(getLineFile(uo->getfnSavedMsg(), 3, 3)));           
      EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); 
        FormEdit (MAXLENGTH_TO, 50, "bcc", HTML2text(getLineFile(uo->getfnSavedMsg(), 4, 4)));         
      EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
        FormEdit (255, 50, "subject", HTML2text(getLineFile(uo->getfnSavedMsg(), 5, 5))); 
      EndTD(); EndTR();
      EndTable ();
      
      if (FileExists (uo->getfnSavedMsg()) == false)
        {
        FormCheckbox ("savemsgsentmail", bool2char(uo->getSaveMsgSentMail()), uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); BR();  
        }
      else
        {                                                
        FormCheckbox ("savemsgsentmail", getLineFile(uo->getfnSavedMsg(), 1, 1), char2bool(getLineFile(uo->getfnSavedMsg(), 1, 1))); WRITE (L->get(L_SAVEMSGSENTMAIL)); BR();
        }
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }
      Label (L->get(L_TEXTTOSEND)); BR();
      if (IsEmpty(getLineFile(uo->getfnSavedMsg(), 6, -1)))
        {
        if (uo->getInsertSignatureInNewMsg() == true) 
          {JS = XString ("\n"); JS += uo->getSignature();} else {JS = XString("");}
        }
      else
        {
        JS = XString (getLineFile(uo->getfnSavedMsg(), 6, -1));
        } 
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", HTML2text((char *)JS.cstr())); BR();
      break;
    case CMD_CM_REPLY:               
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      strncpy (abuf, user, CSMALLBUFFER); strcat (abuf, "@"); strcat (abuf, uo->getMailDomain());
      //Reply to field TO or to field REPLYTO
      if ((uo->getReplyFrom() == true) || 
          ((uo->getReplyFrom() == false) && (strcmp (replyto, "") == 0)))
        {
        //If the FROM field is the same that user@maildomain then use TO field
        if (strstr (from, abuf) == NULL)
          {
          FormEdit (MAXLENGTH_TO, 50, "to", HTML2text(from)); EndTD(); EndTR();
          }
        else
          {
          FormEdit (MAXLENGTH_TO, 50, "to", HTML2text(to)); EndTD(); EndTR();
          }
        }
      else
        {
        //If the REPLYTO field is the same that user@maildomain then use FROM field
        if (strstr (replyto, abuf) == NULL)
          {
          FormEdit (MAXLENGTH_TO, 50, "to", HTML2text(replyto)); EndTD(); EndTR();
          }
        else
          {
          FormEdit (MAXLENGTH_TO, 50, "to", HTML2text(from)); EndTD(); EndTR();
          }
        }
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, 50, "cc", HTML2text(cc));           EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, 50, "bcc", HTML2text(bcc));         EndTD(); EndTR();
      
      JS = XString (L->get (PREFIX_REPLY));
      JS2 = XString (HTML2text(subject)).trim(); 
      p = NCstrstr ((char *)JS2.cstr(), L->get (PREFIX_REPLY));
      if (p == 1) {JS = XString (JS2);} else {JS += (HTML2text(subject));}
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, 50, "subject", (char *)JS.cstr()); EndTD(); EndTR();
      EndTable ();                                                                        
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }      
      Label (L->get(L_TEXTTOSEND)); BR();
      JS2 = XString ("\n"); 
      JS2 += REPLY_BEGIN_LINE;
      if (uo->getInsertSignatureInNewMsg() == true) {JS3 = XString ("\n"); JS3 += uo->getSignature(); JS3 += "\n\n";} else {JS3 = XString("");}      
      JS = XString (REPLY_BEGIN_LINE); JS += firsttext;
      JS.Cut (MAXLENGTH_IN_REPLY);
      JS.replace (XString("\n"), JS2);
      //JS4 = XString (JS3); JS4 += JS;     
      JS4 = XString ("\n\n"); JS4 += JS; JS4 += "\n\n"; JS4 += JS3;
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", HTML2text((char *)JS4.cstr())); BR();
      break;
    case CMD_CM_FORWARD:
      BeginTable (0, 0, 1, 50);
      BeginTR(); BeginTD(); Label (L->get (L_TO));      EndTD(); BeginTD(); 
      FormEdit (MAXLENGTH_TO, 50, "to", ""); EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_CC));      EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, 50, "cc", HTML2text(cc));           EndTD(); EndTR();
      BeginTR(); BeginTD(); Label (L->get (L_BCC));     EndTD(); BeginTD(); FormEdit (MAXLENGTH_TO, 50, "bcc", HTML2text(bcc));         EndTD(); EndTR();
      JS = XString (L->get (PREFIX_FORWARD)); JS += (HTML2text(subject));
      BeginTR(); BeginTD(); Label (L->get (L_SUBJECT)); EndTD(); BeginTD(); 
      FormEdit (255, 50, "subject", (char *)JS.cstr()); EndTD(); EndTR();
      EndTable ();
      FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); BR();  
      if (uo->numAttachs () > 0)
        {
        Label (L->get(L_ATTACHMSG));
        WRITE ("<UL>");
        for (int i = 0; i < uo->numAttachs (); ++i)
          {
          WRITE ("<LI><B>%d. %s %s</B>\n", i + 1, uo->getFileNameAttach(i), uo->getAttachInfo(i));
          }
        WRITE ("</UL>");          
        }
      Label (L->get(L_TEXTTOSEND)); BR();
      JS = XString ("\n\n\n");
      if (uo->getInsertSignatureInNewMsg() == true) {JS += uo->getSignature(); JS += "\n\n";}
      JS += L->get(FORWARDLINE); JS += "\n"; 
      if (strcmp(to, "")     != 0) {JS += L->get(L_TO);      JS += to;      JS += "\n";}
      if (strcmp(subject, "")!= 0) {JS += L->get(L_SUBJECT); JS += subject; JS += "\n";}
      if (strcmp(replyto, "")!= 0) {JS += L->get(L_REPLYTO); JS += replyto; JS += "\n";}
      if (strcmp(from, "")   != 0) {JS += L->get(L_FROM);    JS += from;    JS += "\n";}
      if (strcmp(date, "")   != 0) {JS += L->get(L_DATE);    JS += date;    JS += "\n";}
      JS += "\n"; 
      JS += firsttext;                                                                         
      FormTextArea (uo->getHeightWriteArea(), uo->getWidthWriteArea(), "texttosend", HTML2text((char *)JS.cstr())); BR();      
      break;  
    }
  EndFont();
  EndForm();   
  /*end_silly_table*/WRITE("</TD></TR></TABLE></CENTER>\n");  
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }

void HTML::PrintShowOptions (char *stlang, char *user, char *connid, UserOptions *uo)
  {
  bool iscurrent;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWOPTIONS), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (CMD_OP, stlang, user, connid, uo);
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_OP);
  FormHidden ("cmd", "op");
  FormHidden ("validator", uo->getValidator());
  Label (L->get(L_SHOWOPTIONS)); BR();
  
  BeginTable (5, 0, 0, 25);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); BeginCenter(); FormButton ("op_cancel", L->get(L_CANCEL)); EndCenter(); EndTD();
  BeginTD(); BeginCenter(); FormButton ("op_save",   L->get(L_SAVE));   EndCenter(); EndTD();
  EndTR(); 
  EndTable();
  
  Linea();
  Label (L->get(L_PERSONALNAME)); FormEdit (200, 50, "personalname", uo->getPersonalName()); BR();  
  FormCheckbox ("savemsgsentmail", "1", uo->getSaveMsgSentMail()); WRITE (L->get(L_SAVEMSGSENTMAIL)); FormEdit (50, 20, "sentmailname", uo->getSentMailName()); BR(); 
  FormCheckbox ("replyfrom", "1", uo->getReplyFrom()); WRITE (L->get(L_REPLYFROM)); BR(); 
  FormCheckbox ("forwattachs", "1", uo->getForwAttachs()); WRITE (L->get(L_FORWATTACHS)); BR(); 
  Linea();
  Label (L->get(L_MSGSPERINDEXPAGE)); FormEdit (5, 5, "msgsperindexpage", xltoa(uo->getMsgsPerIndexPage())); BR();
  Label (L->get(L_TRUNCATELENGTHREADINGMSG)); 
  FormEdit (5, 5, "truncatelengthreadingmsg", xltoa(uo->getTruncateLengthReadingMsg())); BR();
  Label (L->get(L_SIZEWRITEAREA)); 
  FormEdit (5, 5, "heightwritearea", xltoa(uo->getHeightWriteArea())); Label (" / ");
  FormEdit (5, 5, "widthwritearea", xltoa(uo->getWidthWriteArea())); BR();
  Label (L->get(L_REFRESHTIME)); 
  FormEdit (5, 5, "refreshtime", xltoa(uo->getRefreshTime())); BR();
  FormCheckbox ("confirmpurge",  "1", uo->getConfirmPurge());  WRITE (L->get(L_CONFIRMPURGE));  BR();
  FormCheckbox ("confirmlogout", "1", uo->getConfirmLogout()); WRITE (L->get(L_CONFIRMLOGOUT)); BR();
  Label (L->get(L_SKIN));
  FormBeginSelect ("skin");
  if (strcmp (uo->getSkin(), "DEFAULT") == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent,  "DEFAULT", "DEFAULT");
  if (strcmp (uo->getSkin(), "OLIVE") == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent, "OLIVE",   "OLIVE");
  if (strcmp (uo->getSkin(), "RED") == 0) {iscurrent = true;} else {iscurrent = false;} FormAddOptionSelect (iscurrent, "RED",   "RED");
  FormEndSelect();
  BR();
  Linea();
  FormCheckbox ("insigninnewmsg", "1", uo->getInsertSignatureInNewMsg()); WRITE (L->get(L_INSIGNINNEWMSG)); BR();    
  Label (L->get(L_SIGNATURE)); BR(); FormTextArea (10, 55, "textsignature", uo->getSignature()); BR();  
  EndForm ();
  EndFont();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
     
void HTML::PrintInvalidPage (char *stlang, char *user, char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_INVALIDPAGE), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  RedLabel (L->get(MSG_INVALIDPAGE)); BR();
  WRITE ("<A HREF=\"%s/mb_index/noop/%s/%s/%s/%d/%d/%d/%ld/\">", PATH_POSTMAN, stlang, user, connid, uo->getValidator(), 
         uo->getPageId(), uo->getLastCmd(), uo->getCurrentPosition());
  WRITE ("<img src=\"%s\" border=0 alt=\"%s\">", L->getI(L_BACK), L->get(L_BACK));
  WRITE ("</A>");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
          
void HTML::PrintBadCookiePage (char *stlang, char *user, char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_BADCOOKIE), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  WRITE ("<CENTER><img src=\"%s\" border=0 alt=\"%s\"></CENTER>", L->getI(L_SECURITYPROBLEM), L->get(L_SECURITYPROBLEM));
  WRITE ("<CENTER>\n");
  RedLabel (L->get(MSG_BADCOOKIE)); BR();
  WRITE ("</CENTER>\n");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }                    
          
void HTML::PrintShowAttachs (char *stlang, char *user, char *connid, UserOptions *uo)
  {
  XString XS;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_SHOWATTACHS), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
        uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraAttachs (stlang, user, connid, uo);
  Linea ();
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));    
  if (uo->numAttachs() > 0)
    {
    BeginFormPost (PATH_POSTMAN);
    FormHidden ("lang", stlang);
    FormHidden ("user", user);
    FormHidden ("connid", connid);
    FormHidden ("cmd", "cm");
    FormHidden ("validator", uo->getValidator());    
    FormHidden ("pageid", uo->getPageId());
    FormHidden ("lastcmd", CMD_AT_SHOW);    
    Label (L->get(L_ATTACHMSG)); BR();    
    for (int i=0; i < uo->numAttachs(); ++i) 
      {
      FormCheckbox ("attnumber", xltoa(i), false); 
      XS = XString (""); XS += xitoa(i + 1); XS += ". "; XS += HTML2text(uo->getFileNameAttach(i)); XS += " "; 
      XS += HTML2text(uo->getAttachInfo(i)); XS += "<BR>\n";
      WRITE ("<B><FONT COLOR=\"%s\">%s</FONT></B>", RED, (char *)XS.cstr());
      }
    BR();
    FormButton ("at_erase", L->get(L_DETACH));
    EndForm ();
    }   
  else
    {
    Label (L->get(L_NO_ATTACHS)); BR();
    }
  Linea ();
  BeginFormPostMultipartFormData (PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "cm");
  FormHidden ("validator", uo->getValidator());  
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", CMD_AT_SHOW);
  Label (L->get(L_ATTACH_NEW)); BR();
  Label (L->get(L_FILE)); FormFile (255, 30, "file", "", "*"); BR();
  FormButton ("at_save", L->get(L_DOATTACH));
  EndForm ();
  EndFont();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
  
void HTML::PrintAddBook_Show (int thecmd, char *stlang, char *user, char *connid, 
                              UserOptions *uo, Addressbook *ab, int sortby)
  {
  TBuffer abuf;
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_ADDBOOK), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (thecmd, stlang, user, connid, uo);
  
  BeginTable (1, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTDColSpan(5); WRITE ("<B>%s: %d</B>\n", L->get(L_NUMOFENTRIES), ab->Count ()); EndTD();
  /////////////// xxxxxxxxxxxxxxxxx uo->getSkinAtribProp("FONT_TABLE"), BLACK, 
  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/0/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_NICKNAME), abuf); EndTD();
  
  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/1/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_FULLNAME), abuf); EndTD();
    
  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/2/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_AB_ADDRESSES), abuf); EndTD();   

  sprintf (abuf, "%s/ab/ab_sort/%s/%s/%s/%d/%d/%d/3/", PATH_POSTMAN, stlang, user, connid, 
           uo->getValidator(), uo->getPageId(), uo->getLastCmd()); 
  BeginTD(); URL (L->get(L_ADDBOOK), abuf); EndTD();     
  
  BeginTD(); WRITE ("<U>%s</U>\n", L->get(L_AB_COMMENTS)); EndTD();
  EndTR();
  ab->SORT(sortby);
  for (int N = 0; N < ab->Count (); ++N)
    {
    WRITE ("<TR valign=baseline bgcolor=\"%s\">", uo->getSkinAtribProp("COLOR_BGFILASADDBOOK"));
    BeginTD();
      Label (xitoa(N + 1));
    EndTD();
    BeginTD(); 
        sprintf (abuf, "%s/ab/ab_addto/%s/%s/%s/%d/%d/%d/%d/", PATH_POSTMAN, 
                 stlang, user, connid, uo->getValidator(), uo->getPageId(), uo->getLastCmd(), N); 
        URL ("TO", abuf);  
    EndTD();
    BeginTD();  
      sprintf (abuf, "%s/ab/ab_addcc/%s/%s/%s/%d/%d/%d/%d/", PATH_POSTMAN, 
               stlang, user, connid, uo->getValidator(), uo->getPageId(), uo->getLastCmd(), N); 
      URL ("CC", abuf); 
    EndTD();  
    BeginTD();
      sprintf (abuf, "%s/ab/ab_addbcc/%s/%s/%s/%d/%d/%d/%d/", PATH_POSTMAN, 
               stlang, user, connid, uo->getValidator(), uo->getPageId(), uo->getLastCmd(), N); 
      URL ("BCC", abuf); 
    EndTD();
    BeginTD(); 
      sprintf (abuf, "%s/ab/ab_editentry/%s/%s/%s/%d/%d/%d/%d/", 
               PATH_POSTMAN, stlang, user, connid, uo->getValidator(), uo->getPageId(), uo->getLastCmd(), N); 
      URLIMAGE (L->get(L_AB_EDITENTRY), abuf, L->getI(L_AB_EDITENTRY));
      URL (L->get(L_EDIT), abuf);
    EndTD();           
    BeginTD(); WRITE (CutSign(ForceSpace(ab->getNickname  (N)), 20, CUTINDICATOR)); EndTD();    
    BeginTD(); WRITE (CutSign(ForceSpace(ab->getFullname  (N)), 20, CUTINDICATOR)); EndTD();
    BeginTD(); WRITE (CutSign(ForceSpace(ab->getAddresses (N)), 20, CUTINDICATOR)); EndTD();
    BeginTD(); WRITE (CutSign(ForceSpace(ab->getFcc       (N)), 20, CUTINDICATOR)); EndTD();
    BeginTD(); WRITE (CutSign(ForceSpace(ab->getComment   (N)), 20, CUTINDICATOR)); EndTD();
    EndTR();
    }
  EndTable ();  
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }         

void HTML::PrintAddBook_Dump (int thecmd, char *stlang, char *user, char *connid, 
                              UserOptions *uo, Addressbook *ab, int sortby)
  {
  int NUM;
  XString as;
  Head ("text/html", "", "");
  ab->SORT(sortby);
  NUM = ab->Count ();
  if (NUM <= 0)
    {
    WRITE ("%s\n", L->get(MSG_NO_ELEMENTS));
    }
  else
    {  
    WRITE ("<PRE>\n");
    for (int N = 0; N < ab->Count (); ++N)
      {
      WRITE ("<BR>#%d ------------------------------------------\n", N + 1);
      WRITE ("<BR>%s: %s\n", L->get(L_AB_NICKNAME),  ab->getNickname  (N));    
      WRITE ("<BR>%s: %s\n", L->get(L_AB_FULLNAME),  ab->getFullname  (N));
      WRITE ("<BR>%s: %s\n", L->get(L_ADDBOOK),   ab->getFcc       (N));         
      WRITE ("<BR>%s: %s\n", L->get(L_AB_COMMENTS),  ab->getComment   (N));
      as = XString (ab->getAddresses (N)); 
      as.replace (",", "<BR>");
      WRITE ("<BR>%s\n", as.cstr());    
      }
    WRITE ("</PRE>\n");      
    }
  }

void HTML::PrintAddBook_EditEntry (int thecmd, char *stlang, char *user, char *connid, 
                                   UserOptions *uo, Addressbook *ab, int numentry)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  Title (L->get(TIT_ADDBOOK), uo->getSkinAtribProp("COLOR_BGBODY"), uo->getSkinAtribProp("COLOR_BGTEXTO"),
         uo->getSkinAtribProp("IMG_BG"), uo->get_use_javascript(), 0, NULL);
  BotoneraComun (thecmd, stlang, user, connid, uo);
  Linea();
  BeginFont (uo->getSkinAtribProp("FONT_MAIN"), uo->getSkinAtribPropI("FONT_MAIN_SIZE"), uo->getSkinAtribProp("COLOR_FONT_MAIN"));  
  BeginFormPost (PATH_POSTMAN);
  FormHidden ("lang", stlang);
  FormHidden ("user", user);
  FormHidden ("connid", connid);
  FormHidden ("cmd", "ab");
  FormHidden ("numentry", xitoa(numentry));
  FormHidden ("validator", uo->getValidator()); 
  FormHidden ("pageid", uo->getPageId());
  FormHidden ("lastcmd", thecmd);    
  FormButton ("ab_saveentry", L->get(L_SAVE));
  if (thecmd == CMD_AB_EDITENTRY) FormButton ("ab_deleentry", L->get(L_DELETE));
  else FormButton ("ab_deleentry", L->get(L_CANCEL));
  BeginTable (1, 2, 1, 100);
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_NICKNAME)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "nickname", HTML2text(ab->getNickname(numentry)));}
  else {FormEdit (255, 50, "nickname", "");}
  EndTD();  
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_FULLNAME)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "fullname", HTML2text(ab->getFullname(numentry)));}
  else {FormEdit (255, 50, "fullname", "");}
  EndTD();  
  EndTR();
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_ADDRESSES)); EndTD();                
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (MAXLENGTH_TO, 50, "addresses", HTML2text(ab->getAddresses(numentry)));} 
  else {FormEdit (MAXLENGTH_TO, 50, "addresses", "");}
  EndTD();  
  EndTR();  
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_ADDBOOK)); EndTD();                
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "fcc", HTML2text(ab->getFcc(numentry)));} 
  else {FormEdit (255, 50, "fcc", "");}
  EndTD();  
  EndTR();    
  BeginTR(uo->getSkinAtribProp("COLOR_BGTABLE"));
  BeginTD(); Label (L->get(L_AB_COMMENTS)); EndTD();
  BeginTD(uo->getSkinAtribProp("COLOR_BGFILASREADINGMSG")); 
  if (numentry > -1) {FormEdit (255, 50, "comments", HTML2text(ab->getComment(numentry)));} 
  else {FormEdit (255, 50, "comments", "");}
  EndTD();  
  EndTR();  
  EndTable ();  
  EndForm ();
  EndFont();
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }           
    
void HTML::PrintDebugPage (char *stlang, char *user, char *connid, UserOptions *uo)
  {
  Head ("text/html", "", "");
  HTMLBegin ();
  WRITE ("DEBUG\n");
  HTMLEnd(uo->getSkinAtribProp("COLOR_BGTABLECOPYRIGHT"));      
  }
  
