/*
Utils.h
*/

#ifndef __UTILS_H
#define __UTILS_H

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <pwd.h>
#include <malloc.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <ctype.h>
#include <syslog.h>
#include <dirent.h>
#include <sys/times.h>                                             

#include "Config.h"
#include "XSocket.h"

#if defined (LINUX)
#include <asm/errno.h>
#elif defined (HP) 
#include <sys/errno.h>
#endif

#include "XString.h"
#include "XVector.h"
typedef XVector<XString> StringList;
typedef XVector<xulong> ULongList;

#define CMD_CONF_ACT        -3    //SPECIALS 
#define CMD_INVALIDPAGE     -2
#define CMD_INVALID         -1
#define CMD_HOME             0    //MISC
#define CMD_LOGIN            1
#define CMD_LOGOUT           2
#define CMD_HELP             3
#define CMD_MG_DUMP          4    //MESSAGE
#define CMD_MG_DISPLAY       5
#define CMD_MG_COPY          6
#define CMD_MG_MOVE          7                           
#define CMD_MG_DELDISPLAY    8
#define CMD_MG_UNDELDISPLAY  9
#define CMD_MB_INDEX        10    //MAILBOX
#define CMD_MB_SHOW         11
#define CMD_MB_CHANGE       12
#define CMD_MB_CREATE       13
#define CMD_MB_DELETE       14
#define CMD_MB_RENAME       15
#define CMD_MB_EXPUNGE      16
#define CMD_FG_DELETE       17    //FLAGS
#define CMD_FG_UNDELETE     18
#define CMD_FG_ANSWERED     19
#define CMD_FG_UNANSWERED   20
#define CMD_FG_FLAGGED      21                           
#define CMD_FG_UNFLAGGED    22
#define CMD_FG_SEEN         23
#define CMD_FG_UNSEEN       24
#define CMD_FG_RECENT       25
#define CMD_FG_UNRECENT     26    
#define CMD_CM              27    //COMPOSE MESSAGE   
#define CMD_CM_SEND         28
#define CMD_CM_CANCEL       29
#define CMD_CM_REPLY        30
#define CMD_CM_FORWARD      31
#define CMD_CM_CLEANALL     32
#define CMD_CM_SAVE         33
#define CMD_OP              34    //OPTIONS
#define CMD_OP_SAVE         35
#define CMD_OP_CANCEL       36
#define CMD_AT_DUMP         37    //ATTACHMENTS
#define CMD_AT_SHOW         38
#define CMD_AT_SAVE         39
#define CMD_AT_ERASE        40
#define CMD_AB              41    //ADDRESSBOOK
#define CMD_AB_SAVEENTRY    42
#define CMD_AB_EDITENTRY    43
#define CMD_AB_SHOWALL      44
#define CMD_AB_FROM_CM      45
#define CMD_AB_DELEENTRY    46
#define CMD_AB_NEWENTRY     47
#define CMD_AB_SORT         48
#define CMD_AB_TAKEADDRESS  49
#define CMD_AB_ADDTO        50
#define CMD_AB_ADDCC        51
#define CMD_AB_ADDBCC       52
#define CMD_MB_DUMP         53
#define CMD_AB_DUMP         54
#define CMD_CHECK_COOKIE    55
#define CMD_NOOP            56
#define CMD_DEBUG           57
#define CMD_MB_SEARCH       58
#define CMD_MB_SORT         59
#define CMD_MG_FG_DELETE     60   
#define CMD_MG_FG_UNDELETE   61   
#define CMD_MG_FG_ANSWERED   62   
#define CMD_MG_FG_UNANSWERED 63  
#define CMD_MG_FG_FLAGGED    64                             
#define CMD_MG_FG_UNFLAGGED  65  
#define CMD_MG_FG_SEEN       66  
#define CMD_MG_FG_UNSEEN     67  
#define CMD_MG_FG_SEARCHED   68  
#define CMD_MG_FG_UNSEARCHED 69  

#define CMD_HELP_LOGIN          0
#define CMD_HELP_INDEXMAILBOX   1
#define CMD_HELP_DISPLAYMSG     2
#define CMD_HELP_SHOWMAILBOXES  3
#define CMD_HELP_COMPOSEMSG     4
#define CMD_HELP_SHOWOPTIONS    5
#define CMD_HELP_ADDBOOK        6
#define CMD_HELP_SHOWATTACHS    7
#define CMD_HELP_ABOUT          8
#define CMD_HELP_AB_EDITENTRY   9
#define CMD_HELP_AB_NEWENTRY   10

#define SP ' '
#define TAB '\t'

#define HTML_CR        "\n"
#define HTML_SP        "&#160;"
#define HTML_MENOR     "&#60;"
#define HTML_MAYOR     "&#62;"
#define HTML_DCOMILLAS "&#34;"
#define HTML_HSP       "&nbsp;"

#define IMAP_ERROR_FATAL (long) 1000

#define INBOX HTML_DCOMILLAS"INBOX"HTML_DCOMILLAS

//#define HTML_VERSION "3.2"
//#define HTML_DOCTYPE "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML "HTML_VERSION" Final//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"
#define HTML_VERSION "4.01"
#define HTML_DOCTYPE "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML "HTML_VERSION" Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"

#define WHITE      "#FFFFFF"
#define BLACK      "#000000"
#define RED        "#FF0000"
#define GREEN      "#00AA00"
#define BLUE       "#0000FF"
#define BLUE2      "#008080"

#define NO_SORT 1000

//FLAGS: only one char!
#define FRECENT      'R'
#define FNEW         'N' 
#define FUNSEEN      'U'
#define FFLAGGED     'F'
#define FANSWERED    'A'
#define FDELETED     'D'
#define FSEARCHED    '@'
#define FEMPTY       '-'

#define POS_FLAG_DELETED   0
#define POS_FLAG_ANSWERED  1
#define POS_FLAG_NEW       2
#define POS_FLAG_IMPORTANT 3
#define POS_FLAG_SEARCHED  4

#define CMAXHBUFFER 20000
typedef char THBuffer[CMAXHBUFFER + 1];
#define CMAXBUFFER 2000
typedef char TBuffer[CMAXBUFFER + 1];
#define CSMALLBUFFER 200
typedef char TSBuffer[CSMALLBUFFER + 1];
#define CMAXFILENAME 255
typedef char TFileName[CMAXFILENAME + 1];
#define MAXLENGTHLINE 500
typedef char LINEA [MAXLENGTHLINE + 1];
#define CMAXIPNAME 50
typedef char IPNAME [CMAXIPNAME + 1];

#define MAXLENGTHSKIN 50
typedef int FONTSIZE;
#define MAXLENGTHFONTNAME 50
typedef char FONTNAME [MAXLENGTHFONTNAME + 1];
#define MAXLENGTHCOLORHTML 7
typedef char COLORHTML [MAXLENGTHCOLORHTML + 1];

void initStr (char *thestr);
void initStr (char *thestr, const char *consstr);
char *Cut (char *pc, int maxlong);
bool CreateDir (char *path, mode_t mode);
bool DeleteFile (const char *fname);
int CreateAndLockFile (char *fn);
void EscribePIDenLockFile (int fd);
void UnlockAndCloseFile (int fd);
int IsLocked (char *fn, int *fd);
int GetPIDFromLockFile (int fd);
long GetTimeFromLockFileOpening (char *fn);
long GetTimeFromLockFile (int fd);
void SaveFileDebug (char *fn, char *pc, int len);
void SaveFileDebug (char *fn, char *pc);
char *ReturnUsersDir (const char *basepath, const char *user, const char *imapserver);
char *ReturnUserLockFileName (const char *user);
char *ReturnUserAttachsDir (const char *basepath, const char *user, const char *imapserver);
char *ReturnSignatureFileName (const char *basepath, const char *user, const char *imapserver);
char *ReturnUserOptionsFileName (const char *basepath, const char *user, const char *imapserver);
char *ReturnUserAddressbookFileName (const char *basepath, const char *user, const char *imapserver);
char *ReturnUserSavedMsgFileName (const char *basepath, const char *user, const char *imapserver);
void QuitaRetornoCarroDeLinea (char *linea);
char *xstrdup (char const *str);
void Randomize ();
int Rand (int max);
char *MakeTmpName (char *fntmp);
char *getExtensionFromFile (const char *fn);
bool FileExists (const char *fn);
int FileSize (const char *fn);
time_t FileDate (char *fn);
char *xucase (char *s);
char *xlcase (char *s);
int TranslateCmd (char *stcmd);
int TranslateHelpCmd (char *sthelp);
char *xitoa (int n);
char *xltoa (long n);
int xatoidef (const char *stnum, int def);
long int xatoldef (const char *stnum, long int def);
char *make_session_id (const char *imapserver);
char *extract_imapserver_from_connid (const char *connid);
char *make_root_socket (void);
char *make_session_socket (char *username, char *connid);
void ini (char *connid, char *stlang, char *stcmd, char *allparms, char *allparms2,
          char *parm1, char *parm2, char *parm3, char *parm4);
int NCstrstr (const char *linea, const char *token);
int Cstrstr (const char *linea, const char *token);
void OPENLOG (void);
void CLOSELOG (void);
void LOG (char *fmt, ...);
void LOG (long i);
void DEBUG (char *fmt, ...);
void DEBUG (int imsg);
char *FormatCountBytes (long count);
char *FormatCountBytes1Dec (long count);
void Redirect (char *newurl);
void ErrorPage (char *error);
void CGIErrorPageConnectionRefused (char *lang);
void PrintHelpPage (char *topic);
bool CopyFile (char *oldfile, char *newfile);
bool CopyFileOverWrite (char *oldfile, char *newfile);
char *replace (char *src, char *why, char *forwhy);
char *HTML2text (char *ast);
bool IsEmpty (char *pc);
bool IsNull (char *pc);
bool IsGoodValidator (int cmd, int validator, int validatorgood);
void Ltrim (char *linea);
void Rtrim (char *linea);
void trim  (char *linea);
char *ForceSpace (char *value);
char *CutSign (char *pc, int maxlong, char *signal);
char *getLineFile (char *flname, int x, int y);
void CreaFileSavedMsg (char *fn);
bool ConcatLineToLineOfFile (char *fn, int numlinea, char *whatadd, char *sep, int MAX);
int CountChar (char *string, char tocount);
pid_t WritePidFile (void);
void DeletePidFile (void);
void DelAllFilesFromDir (char *thedir);
char *getMailcapTipoFromFilename (const char *fn, const char *fnmailcap);
void PrintHomePage (char *lang);
void DumpHTMLPage (char *fn);
long min (long i,long j);
void TICK (char *label);
void PChar2SL (char *chunk, StringList *SL);
void ResaltaURLsEnText (char *txt, XSocket *aXS, int MaxLongLine);
bool IsMultiplo (int dividendo, int divisor);
void xscanf (const char *linea, 
            char *c0,
            int *int1, int *int2, int *int3, 
            char *c1, char *c2, char *c3, char *c4, char *c5, char *c6, char *c7);
char *ProcesaDate (char *tmp);
int GetBufferFromFile (char *afile, char *abuf, int maxlength);
char *bool2char (bool b);
bool char2bool (char *c);
char *DisplayFlags (const char *flags, bool clickable, const char *molde, int show_way);
                          
#endif
